;;;; Copyright (c) 2004 David Lichteblau <david@lichteblau.com>
;;;;
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(in-package :jdii)

(define-class jdi:virtual-machine () ())
(define-class jdi:type () ())
(define-class jdi:primitive-type (jdi:type) (jdi:name jdi:signature))
(define-class jdi:reference-type (jdi:type) ())
(define-class jdi:array-type (jdi:reference-type) ())
(define-class jdi:class-type (jdi:reference-type) ())
(define-class jdi:interface-type (jdi:reference-type) ())
(define-class jdi:member () ())
(define-class jdi:field (jdi:member) ())
(define-class jdi:method (jdi:member) ())
(define-class jdi:object () ())
(define-class jdi:array (jdi:object) ())
(define-class jdi:thread-group (jdi:object) (jdi:name))
(define-class jdi:thread (jdi:object) (jdi:name))
(define-class jdi:frame () (jdi:location))
(define-class jdi:location ()
  (jdi:type method-id jdi:line-number jdi:code-index))
(define-class jdi:local-variable () (jdi:name jdi:signature))

(define-class jdi:event-request ()
  ((jdi:suspend-policy :initform nil)
   (modifiers :initform '())
   (jdi:plist :initform '())
   vm
   request-id))
(define-class jdi:watchpoint-request (jdi:event-request)
  (jdi:field))

(define-class jdi:breakpoint-request (jdi:event-request)
  (jdi:location))
(define-class jdi:class-prepare-request (jdi:event-request) ())
(define-class jdi:class-unload-request (jdi:event-request) ())
(define-class jdi:exception-request (jdi:event-request)
  (jdi:reference-type jdi:caughtp jdi:uncaughtp))
(define-class jdi:field-access-request (jdi:watchpoint-request) ())
(define-class jdi:field-modification-request (jdi:watchpoint-request) ())
(define-class jdi:method-entry-request (jdi:event-request) ())
(define-class jdi:method-exit-request (jdi:event-request) ())
(define-class jdi:step-request (jdi:event-request)
  (jdi:thread jdi:size jdi:depth))
(define-class jdi:thread-death-request (jdi:event-request) ())
(define-class jdi:thread-start-request (jdi:event-request) ())
(define-class jdi:vm-death-request (jdi:event-request) ())
(define-class jdi:vm-disconnected-request (jdi:event-request) ())
(define-class jdi:vm-start-request (jdi:event-request) ())

(define-class modifier () ())
(define-class location-modifier (modifier) (location))
(define-class exception-only-modifier (modifier) (reference-type caughtp uncaughtp))
(define-class field-modifier (modifier) (field))
(define-class step-modifier (modifier) (thread size depth))

(defmethod initialize-instance :after ((instance jdi:breakpoint-request) &key)
  (unless (jdi:location instance)
    (error "breakpoint location not specified"))
  (push (make-instance 'location-modifier :location (jdi:location instance))
	(modifiers instance)))

(defmethod initialize-instance :after ((instance jdi:exception-request) &key)
  (with-slots (jdi:reference-type jdi:caughtp jdi:uncaughtp) instance
    (unless jdi:reference-type (error "exception type not specified"))
    (unless jdi:caughtp (error "`catch' mode not specified"))
    (unless jdi:uncaughtp (error "`uncatch' mode not specified"))
    (push (make-instance 'exception-only-modifier
	    :reference-type jdi:reference-type
	    :caughtp jdi:caughtp
	    :uncaughtp jdi:uncaughtp)
	  (modifiers instance))))

(defmethod initialize-instance :after ((instance jdi:watchpoint-request) &key)
  (unless (jdi:field instance) (error "field not specified"))
  (push (make-instance 'field-modifier :field (jdi:field instance))
	(modifiers instance)))

(defmethod initialize-instance :after ((instance jdi:step-request) &key)
  (with-slots (jdi:thread jdi:size jdi:depth) instance
    (unless jdi:thread (error "thread not specified in step request"))
    (unless jdi:size (error "step size not specified"))
    (unless jdi:depth (error "step depth not specified"))
    (push (make-instance 'step-modifier
	    :thread jdi:thread
	    :size jdi:size
	    :depth jdi:depth)
	  (modifiers instance))))

(define-class jdi:event () (jdi:request))
(define-class jdi:thread-event (jdi:event)
  (jdi:thread))
(define-class jdi:locatable-event (jdi:thread-event)
  (jdi:location))
(define-class jdi:watchpoint-event (jdi:locatable-event)
  (jdi:field jdi:object))

(define-class jdi:breakpoint-event (jdi:locatable-event) ())
(define-class jdi:class-prepare-event (jdi:thread-event)
  (jdi:class))
(define-class jdi:class-unload-event (jdi:event)
  (jdi:signature))
(define-class jdi:exception-event (jdi:locatable-event)
  (jdi:exception jdi:catch-location))
(define-class jdi:field-access-event (jdi:watchpoint-event) ())
(define-class jdi:field-modification-event (jdi:watchpoint-event)
  (jdi:value))
(define-class jdi:method-entry-event (jdi:locatable-event) ())
(define-class jdi:method-exit-event (jdi:locatable-event) ())
(define-class jdi:step-event (jdi:locatable-event) ())
(define-class jdi:thread-death-event (jdi:thread-event) ())
(define-class jdi:thread-start-event (jdi:thread-event) ())
(define-class jdi:vm-death-event (jdi:event) ())
(define-class jdi:vm-disconnected-event (jdi:thread-event) ())
(define-class jdi:vm-start-event (jdi:thread-event) ())

(define-class jdi:event-set () (jdi:suspend-policy jdi:events))

(defgeneric jdi:name (object))
(defgeneric jdi:signature (class))


;;; VIRTUAL-MACHINE

(defgeneric jdi:resume (vm-or-thread))
(defgeneric jdi:suspend (vm-or-thread))

(defgeneric jdi:all-classes (vm))
(defgeneric jdi:all-threads (vm))
(defgeneric jdi:base-directory (vm))
(defgeneric jdi:bootstrap-class-path (vm))
(defgeneric jdi:classes-by-name (vm name))
(defgeneric jdi:classes-by-signature (vm signature))
(defgeneric jdi:class-path (vm))
(defgeneric jdi:description (vm))
(defgeneric jdi:dispose (vm))
(defgeneric jdi:exit (vm rc))
(defgeneric jdi:get-capability (vm name))
(defgeneric jdi:id-sizes (vm))
(defgeneric jdi:jdwp-version (vm))
(defgeneric jdi:read-event-set (vm &key wait))
(defgeneric jdi:top-level-thread-groups (vm))
(defgeneric jdi:version (vm))


;;; REFERENCE-TYPE

(defgeneric jdi:all-fields (class))
(defgeneric jdi:all-methods (class &key exclude-overriden))
(defgeneric jdi:fields (class))
(defgeneric jdi:field-value (field class))
(defgeneric jdi:field-values (fields class))
(defgeneric jdi:interfaces (class))
(defgeneric jdi:methods (class))
(defgeneric jdi:superclass (class))	;CLASS-TYPE only
(defgeneric jdi:source-file (class))

(defmethod jdi:name ((class jdi:reference-type))
  (jni-to-java (jdi:signature class)))

(defgeneric jdi:subclasses (class))
(defgeneric jdi:subinterfaces (class))
(defgeneric jdi:superinterfaces (class))

(defgeneric jdi:component-signature (array-class))
(defgeneric jdi:component-type (array-class))
(defgeneric jdi:component-type-name (array-class))


;;; MEMBER

(defgeneric jdi:declaring-type (member))

(defmethod jdi:argument-type-names ((method jdi:method))
  (parse-method-descriptor (jdi:signature method)))

(defmethod jdi:argument-types ((method jdi:method))
  ;; XXX
  (error "not implemented")
  )

(defmethod jdi:return-type-name ((method jdi:method))
  (nth-value 1 (parse-method-descriptor (jdi:signature method))))

(defgeneric jdi:modifiers (member))

(defgeneric jdi:local-variables (method))
(defgeneric jdi:guess-local-variables (method &optional class))

(defgeneric jdi:package-private-p (member))
(defgeneric jdi:privatep (member))
(defgeneric jdi:protectedp (member))
(defgeneric jdi:publicp (member))
(defgeneric jdi:finalp (member))
(defgeneric jdi:staticp (member))
(defgeneric jdi:syntheticp (member))
(defgeneric jdi:transientp (member))
(defgeneric jdi:volatilep (member))
(defgeneric jdi:abstractp (member))
(defgeneric jdi:nativep (member))
(defgeneric jdi:synchronizedp (member))


;;; JDWP constants

(defconstant +typetag-class+ 1)
(defconstant +typetag-interface+ 2)
(defconstant +typetag-array+ 3)

(defconstant +classstatus-verified+ 1)
(defconstant +classstatus-prepared+ 2)
(defconstant +classstatus-initialized+ 3)
(defconstant +classstatus-error+ 8)


;;; THREAD

(defgeneric jdi:frames (thread))
(defgeneric jdi:group (thread))
(defgeneric jdi:status (thread))
(defgeneric jdi:suspendedp (thread))


;;; THREAD-GROUP

(defgeneric jdi:parent (group))
(defgeneric jdi:child-threads (group))
(defgeneric jdi:child-groups (group))


;;; ARRAY

(defgeneric jdi:length (array))
(defgeneric jdi:data (array &optional start end))
(defgeneric jdi:elt (array index))


;;; FRAME

(defgeneric jdi:get-values (frame locals))
(defgeneric jdi:this-object (frame))


;;; LOCAL-VARIABLE

(defmethod jdi:type-name ((local jdi:local-variable))
  (jni-to-java (jdi:signature local)))


;;; ...

(defmacro jdi:with-virtual-machine ((var vm) &body body)
  `(let ((,var ,vm))
     (unwind-protect
	 (progn ,@body)
       (jdi:dispose ,var))))
