;;;; copyright (c) 1999 David Lichteblau <david@lichteblau.com>
;;;;
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(in-package :jdii)

(defclass queue ()
    ((front :initform '() :accessor queue-front)
     (rear :initform '() :accessor queue-rear))
  (:documentation
   "A queue (QUEUE-INSERT, QUEUE-DELETE). This class is not thread-safe.
See also: SYNCHRONIZED-QUEUE."))

(defun make-simple-queue ()
  (make-instance 'queue))

(defgeneric queue-insert (queue frob)
  (:documentation "Enqueue object FROB."))

(defmethod queue-insert ((queue queue) frob)
  (let ((new-rear (list frob)))
    (if (queue-rear queue)
	(setf (cdr (queue-rear queue)) new-rear)
	(setf (queue-front queue) new-rear))
    (setf (queue-rear queue) new-rear))
  frob)

(defgeneric queue-delete (queue)
  (:documentation "Dequeue the next object, or return NIL if QUEUE is empty."))

(defmethod queue-delete ((queue queue))
  (prog1
      (pop (queue-front queue))
    (unless (queue-front queue)
      (setf (queue-rear queue) nil))))

(defgeneric queue-insert-front (queue frob)
  (:documentation
   "Prepend object FROB.  (Use QUEUE-INSERT for ordinary enqueueing.)"))

(defmethod queue-insert-front ((queue queue) frob)
  (push frob (queue-front queue))
  (unless (queue-rear queue)
    (setf (queue-rear queue) (queue-front queue)))
  frob)

(defgeneric %queue-listen (queue)
  (:documentation "Is QUEUE non-empty?  This function is not thread-safe;
use QUEUE-LISTEN instead."))

(defmethod %queue-listen ((queue queue))
  (not (null (queue-front queue))))

(defgeneric queue-listen (queue)
  (:documentation "Is QUEUE non-empty?"))

(defmethod queue-listen ((queue queue))
  (%queue-listen queue))

(defgeneric queue-peek (queue)
  (:documentation
   "Return the object QUEUE-DELETE would dequeue, or NIL if QUEUE is empty."))

(defmethod queue-peek ((queue queue))
  (if (queue-listen queue)
      (car (queue-front queue))
      nil))

(defgeneric clear-queue (queue)
  (:documentation "Remove all elements waiting in QUEUE."))

(defmethod clear-queue ((queue queue))
  (setf (queue-front queue) '()
	(queue-rear queue) nil)
  nil)
