;;;; Copyright (c) 2004 David Lichteblau <david@lichteblau.com>
;;;;
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(in-package :decaf)

(defmethod display ((file file-page) (frame decaf) *standard-output*)
  (terpri)
  (formatting-header (t :class)
    (with-text-style (t (make-text-style :sans-serif :bold :large))
      ;; (present (name file))
      (princ (name file))))
  (fresh-line)
  (with-open-file (s (name file) :if-does-not-exist nil)
    (unless s
      (write-line "File not found.")
      (return-from display))
    (with-text-style (t (make-text-style :sans-serif :italic :normal))
      (write-string "Last changed: "))
    (multiple-value-bind
	(second minute hour date month year)
	(decode-universal-time (file-write-date (name file)))
      (format t "~4D-~2,'0D-~2,'0D ~2,'0D:~2,'0D:~2,'0D~%"
	      year month date hour minute second))
    #+(or)
    (let ((climi::*pane-realizer* *DEFAULT-FRAME-MANAGER*))
    (terpri)
      (assert climi::*pane-realizer*)
      (with-output-as-gadget (t)
	(make-pane 'push-button-pane
		   :label "Open in Emacs"
		   :activate-callback
		   (lambda (button)
		     (declare (ignore button))
		     (cond
		       ((null (find-package :lep))
			 (write-line "Sorry, ELI not loaded."))
		       ((not (funcall (find-symbol "LEP-IS-RUNNING" :lep)))
			 (write-line "Not connected to Emacs."))
		       (t
			 (funcall (find-symbol "FIND-FILE-IN-EDITOR" :lep)
				  :pathname (name file))))))))
    (fresh-line)
    (terpri)
    (when (from file)
      (let ((p (make-instance 'file-page
		 :name (name file)
		 :from nil
		 :to (to file)
		 :highlight (highlight file))))
	(with-output-as-presentation (t p 'file-page)
	  (with-text-style (t (make-text-style :sans-serif :italic :normal))
	    (write-line "...more lines...")))))
    (terpri)
    (formatting-table (t :x-spacing '(2 :character))
      (for ((line = (read-line s nil nil))
	    (i :from 1)
	    :while line)
	(with-slots (from to highlight) file
	  (with-drawing-options
	      ;; XXX das hatte ich ja gerne als Hintergrundfarbe, wie in
	      ;; einer HTML-Tabelle.  Wie geht das?
	      (t :ink (if (member i highlight) +red+ +foreground-ink+))
	    (when (and (or (null from) (<= from i))
		       (or (null to) (< i to)))
	      (with-output-as-presentation (t (list file i) 'line-of-code)
		(formatting-row ()
		  (formatting-cell (t :align-x :right)
		    (princ i))
		  (formatting-cell ()
		    (with-text-style (t (make-text-style :fixed nil nil))
		      (write-string line))))))))))
    (fresh-line)
    (when (to file)
      (let ((p (make-instance 'file-page
		 :name (name file)
		 :from (from file)
		 :to nil
		 :highlight (highlight file))))
	(with-output-as-presentation (t p 'file-page)
	  (with-text-style (t (make-text-style :sans-serif :italic :normal))
	    (write-line "...more lines...")))))))
