;;;; Copyright (c) 2004 David Lichteblau <david@lichteblau.com>
;;;;
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(in-package :decaf)

(defun display-array (object frame)
  (declare (ignore frame))
  (with-text-style (t (make-text-style :sans-serif :italic nil))
    (write-string "Length: "))
  (write (jdi:length object))
  (terpri)
  (terpri)
  (terpri)

  (surrounding-output-with-border (t :shape :underline)
    (with-text-style (t (make-text-style :sans-serif :bold :normal))
      (write-string "Array Data:")))
  (fresh-line)
  (formatting-table (t :x-spacing '(1 :character))
    (let ((data (jdi:data object))
	  (i 0)
	  (objectsp
	   (member (elt (jdi:signature (jdi:reference-type object)) 1)
		   '(#\L #\[))))
      (jdii::while (< i (length data))
	(flet ((present1 ()
		 (when (< i (length data))
		   (let ((elt (elt data i)))
		     (typecase elt
		       (integer (format t "~2,'0X" elt))
		       (t (present elt))))
		   (incf i))))
	  (formatting-row ()
	    (formatting-cell (t :align-x :right)
	      (write i))
	    (formatting-cell ()
	      (cond
		(objectsp
		  (present1))
		(t
		  (dotimes (x 8) (present1) (write-char #\space))
		  (write-char #\space)
		  (dotimes (x 8) (present1) (write-char #\space))
		  (write-char #\space)
		  (dotimes (x 8) (present1) (write-char #\space))
		  (write-char #\space)
		  (dotimes (x 8) (present1) (write-char #\space)))))))))))

(defun display-nonarray (object frame &optional (underlinep t))
  (let ((class (jdi:reference-type object)))
    (formatting-table (t :x-spacing '(1 :character))
      (format-field-rows frame
			 class
			 "Static Fields"
			 #'jdi:staticp
			 t
			 :green-dot
			 underlinep)
      (write-empty-line)
      (format-field-rows frame
			 object
			 "Instance Fields"
			 (complement #'jdi:staticp)
			 t
			 :green-dot
			 underlinep))))

(defmethod display ((object jdi:object) (frame decaf) *standard-output*)
  (terpri)
  (formatting-header (t :green)
    (with-text-style (t (make-text-style :sans-serif :bold :large))
      (present object))
    (fresh-line))
  (terpri)
  (with-text-style (t (make-text-style :sans-serif :italic nil))
    (write-string "Instance of: "))
  (present-class (jdi:reference-type object))
  (terpri)
  (if (typep object 'jdi:array)
      (display-array object frame)
      (progn
	(terpri)
	(terpri)
	(display-nonarray object frame)))
  ;; make scroll bar behave: 
  (terpri))
