;;;; Copyright (c) 2003,2004 David Lichteblau <david@lichteblau.com>
;;;;
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(in-package :dlisp)

(defun nanp (float)
  (excl::nan-p float))

(defun |java/lang/Double.doubleToRawLongBits(D)| (d)
  (multiple-value-bind (a b c d)
      (excl:double-float-to-shorts d)
    (canonicalize-long
     (logior (ash a 48)
	     (ash b 32)
	     (ash c 16)
	     d))))

(defun |java/lang/Double.longBitsToDouble(J)| (l)
  (setf l (canonicalize-n-unsigned l 64))
  (excl:shorts-to-double-float
   (ldb (byte 16 48) l)
   (ldb (byte 16 32) l)
   (ldb (byte 16 16) l)
   (ldb (byte 16 00) l)))

(defun |java/lang/Float.floatToRawIntBits(F)| (f)
  (multiple-value-bind (a b)
      (excl:single-float-to-shorts f)
    (canonicalize-int
     (logior (ash a 16) b))))

(defun |java/lang/Float.intBitsToFloat(I)| (i)
  (setf i (canonicalize-n-unsigned i 32))
  (excl:shorts-to-single-float
   (ldb (byte 16 16) i)
   (ldb (byte 16 00) i)))

(defun read-sequence-partial-fill (seq stream start end)
  (read-sequence seq stream :start start :end end :partial-fill t))

(defun get-host-by-address (ipaddr-array)
  (socket:ipaddr-to-hostname (array-to-ipaddr ipaddr-array)))

(defun get-host-by-name (name)
  (ipaddr-to-array (socket:lookup-hostname name)))

(defun make-socket (inaddr port)
  (socket:make-socket :remote-host (array-to-ipaddr inaddr) :remote-port port))

(defun socket-stream (socket &key (element-type '(signed-byte 8)))
  (assert (equalp element-type '(unsigned-byte 8)))
  socket)

(defconstant +socket-error+ 'excl:socket-error)

(defun set-socket-option (socket option value)
  (let ((option
	 (ecase option
	   (#.+plainsocketimpl-keep-alive+ :keepalive)
	   (#.+plainsocketimpl-nodelay+ :nodelay))))
    (socket:set-socket-options socket option value)))

(defun gc (&optional full)
  (excl:gc (and full t)))

(defun string-to-utf-8 (string)
  (excl:string-to-octets string :external-format :utf8 :null-terminate nil))

(defun utf-8-to-string (bytes)
  (excl:octets-to-string bytes :external-format :utf8 :end (length bytes)))

(defun decode-pseudo-utf-8 (lisp-string)
  (map 'vector #'char-code lisp-string))

(defun compile-labeled-function (name lambda-expression)
  (funcall
   (compile nil `(lambda () (excl:named-function ,name ,lambda-expression)))))

(defun disable-floating-point-traps ()
  (warn "cannot disable floating point traps"))

(defun enable-floating-point-traps ())

(defun directoryp (pathname)
  (excl:file-directory-p pathname))

(defun wait-for-input-available (fd)
  (mp:wait-for-input-available fd :wait-function #'excl:read-no-hang-p))

(defun add-fd-handler (stream direction function)
  (assert (eq direction :input))
  (mp:process-run-function (format nil "fd handler for ~A" stream)
    (lambda ()
      (while (open-stream-p stream)
	(wait-for-input-available stream)
	(funcall function)))))

(defun remove-fd-handler (handler)
  (mp:process-kill handler)
  (while (member handler sys:*all-processes*)
    (mp:process-allow-schedule)))
