;;;; Copyright (c) 2003,2004 David Lichteblau <david@lichteblau.com>
;;;;
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(in-package :dlisp)

(eval-when (:compile-toplevel :load-toplevel :execute)
  (require "comm"))

(defun nanp (float)
  (system::nan-p float))

(fli:define-c-union my-double
  (value :double)
  (decoded-value (:struct (a :int) (b :int))))

(defun decode-double (double)
  (let ((my-double (fli:allocate-foreign-object :type 'my-double)))
    (unwind-protect
	(progn
	  (setf (fli:foreign-slot-value my-double 'value) double)
	  #-little-endian
	  (values
	   (fli:foreign-slot-value my-double '(decoded-value a))
	   (fli:foreign-slot-value my-double '(decoded-value b)))
	  #+little-endian
	  (values
	   (fli:foreign-slot-value my-double '(decoded-value b))
	   (fli:foreign-slot-value my-double '(decoded-value a))))
      (fli:free-foreign-object my-double))))

(defun encode-double (hi lo)
  #+little-endian (rotatef hi lo)
  (let ((my-double (fli:allocate-foreign-object :type 'my-double)))
    (unwind-protect
	(progn
	  (setf (fli:foreign-slot-value my-double '(decoded-value a)) hi)
	  (setf (fli:foreign-slot-value my-double '(decoded-value b)) lo)
	  (fli:foreign-slot-value my-double 'value))
      (fli:free-foreign-object my-double))))

(defun |java/lang/Double.doubleToRawLongBits(D)| (d)
  (multiple-value-bind (hi lo)
      (decode-double d)
    (canonicalize-long (logior (ash hi 32) lo))))

(defun |java/lang/Double.longBitsToDouble(J)| (l)
  (encode-double
   (canonicalize-int (ldb (byte 32 32) l))
   (canonicalize-n-unsigned (ldb (byte 32 0) l) 32)))

(fli:define-c-union my-float
  (value :float)
  (decoded-value :int))

(defun %decode-float (float)
  (let ((my-double (fli:allocate-foreign-object :type 'my-float)))
    (unwind-protect
	(progn
	  (setf (fli:foreign-slot-value my-float 'value) float)
	  (fli:foreign-slot-value my-float 'decoded-value))
      (fli:free-foreign-object my-float))))

(defun encode-float (int)
  (let ((my-float (fli:allocate-foreign-object :type 'my-float)))
    (unwind-protect
	(progn
	  (setf (fli:foreign-slot-value my-float 'decoded-value) int)
	  (fli:foreign-slot-value my-float 'value))
      (fli:free-foreign-object my-float))))

(defun |java/lang/Float.floatToRawIntBits(F)| (f)
  (canonicalize-int (%decode-float f)))

(defun |java/lang/Float.intBitsToFloat(I)| (i)
  (encode-float i))

(defun read-sequence-partial-fill (seq stream start end)
  (%read-sequence-partial-fill seq stream start end))

(defun get-host-by-address (ipaddr-array)
  (comm:get-host-entry (array-to-ipaddr ipaddr-array) :fields '(:name)))

(defun get-host-by-name (name)
  (ipaddr-to-array (comm:get-host-entry name)  :fields '(:address)))

(defun make-socket (inaddr-array port)
  (comm:open-tcp-stream (array-to-ipaddr inaddr) port
			:element-type '(signed-byte 8)))

(defun socket-stream (socket)
  socket)

(defconstant +socket-error+ 'comm::socket-error)

(defun set-socket-option (socket option value)
  (ecase option
    (#.+plainsocketimpl-keep-alive+)
    (#.+plainsocketimpl-nodelay+))
  (warn "don't know how to set socket options"))

(defun gc (&optional full)
  (declare (ignore full))
  (format *trace-output* "~&; Don't know how to force GC.~%")
  (force-output *trace-output*))

(defun string-to-utf-8 (string)
  (external-format:encode-lisp-string string :utf-8))

(defun utf-8-to-string (bytes)
  (external-format:decode-external-string bytes :utf-8))

(defun decode-pseudo-utf-8 (lisp-string)
  (map 'vector #'char-code lisp-string))

(defun compile-labeled-function (name lambda-expression)
  (fake-labeled-function name lambda-expression))

(defun disable-floating-point-traps ()
  (warn "cannot disable floating point traps"))

(defun enable-floating-point-traps ())

;;; XXX das ist nicht gerade elegant.
;;; XXX thread safety
(defun directoryp (pathname)
  (let ((pwd (HARLEQUIN-COMMON-LISP:GET-WORKING-DIRECTORY)))
    (unwind-protect
        (handler-case
            (progn
              (HARLEQUIN-COMMON-LISP:CHANGE-DIRECTORY pathname)
              t)
          (file-error () nil))
      (HARLEQUIN-COMMON-LISP:CHANGE-DIRECTORY pwd))))
