(in-package :dlisp)

(defconstant 2^32 (expt 2 32))
(defconstant 2^31 (expt 2 31))
(defconstant +int-min+ (- 2^31))
(defconstant +int-max+ (1- 2^31))
(defconstant +int-range+ 2^32)

(defconstant 2^64 (expt 2 64))
(defconstant 2^63 (expt 2 63))
(defconstant +long-min+ (- 2^63))
(defconstant +long-max+ (1- 2^63))
(defconstant +long-range+ 2^64)

#+optimized-int-arithmetic
(declaim (optimize (speed 3) (safety 0)))

(defun canonicalize-int (n)
  #+(or)
  (loop
    (setf n (cond
	      ((> n +int-max+) (- n +int-range+))
	      ((< n +int-min+) (+ n +int-range+))
	      (t (return n)))))
  (let ((k (nth-value 1 (round n 2^32))))
    (if (eql k 2^31)
	(- k)
	k)))

#+(and sbcl optimized-int-arithmetic)
(define-compiler-macro canonicalize-int (&whole whole form)
  (if (listp form)
      `(the (signed-byte 32)
	 (,(car form)
	  ,@(loop
		for arg in (cdr form)
		collect `(the (signed-byte 32) ,arg))))
      whole))

(defun canonicalize-long (n)
  #+(or)
  (loop
    (setf n (cond
	      ((> n +long-max+) (- n +long-range+))
	      ((< n +long-min+) (+ n +long-range+))
	      (t (return n)))))
  (let ((k (nth-value 1 (round n 2^64))))
    (if (eql k 2^63)
	(- k)
	k)))

(defun canonicalize-n (n size)
  #+(or)
  (loop
    (setf n (cond
	      ((> n (1- (expt 2 (1- size)))) (- n (expt 2 size)))
	      ((< n (- (expt 2 (1- size)))) (+ n (expt 2 size)))
	      (t (return n)))))
  (let ((k (nth-value 1 (round n (expt 2 size)))))
    (if (eql k (expt 2 (1- size)))
	(- k)
	k)))

(defun canonicalize-n-unsigned (n size)
  #+(or)
  (let ((m (expt 2 size)))
    (loop
      (setf n (cond
		((>= n m) (- n m))
		((< n 0) (+ n m))
		(t (return n))))))
  (nth-value 1 (floor n (expt 2 size))))

#+optimized-int-arithmetic
(declaim (inline int-+ int-- 0-int int-* int-truncate int-ash))

(defun int-+ (a b) (canonicalize-int (+ a b)))
(defun int-- (a b) (canonicalize-int (- a b)))
(defun 0-int (a) (canonicalize-int (- a)))
(defun int-* (a b) (canonicalize-int (* a b)))
(defun int-truncate (a b) (canonicalize-int (truncate a b)))
(defun int-ash (a n) (canonicalize-int (ash a n)))

(defun long-+ (a b) (canonicalize-long (+ a b)))
(defun long-- (a b) (canonicalize-long (- a b)))
(defun 0-long (a) (canonicalize-long (- a)))
(defun long-* (a b) (canonicalize-long (* a b)))
(defun long-truncate (a b) (canonicalize-long (truncate a b)))
(defun long-ash (a n) (canonicalize-long (ash a n)))
